/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einen aus mehreren Phasen bestehenden Arbeitsauftrag
*/
function PhasesTask(
    nodeParent, // HTML-Element, in dem alles angezeigt werden soll
    data        // JSON mit Informationen über die Phasen
) {
    nodeParent.innerHTML = '';
    // Schaltfläche für Lizenzinfos
    new ScoTools.InfoPageLink();
    // Platz für Meldungen
    this.msgPlace = new ScoTools.MessagePlace(true);
    this.msgPlace.appendAsChild(nodeParent);
    // Verbindung mit Lernplattform aufnehmen
    this.connect();
    // Zeitpunkt des letzten Phasenstarts laden
    this._time = (+this.read('suspendData')) * 1000;
    if (!Number.isFinite(this._time))
        this._time = (new Date()).getTime();
    // Phasen laden
    this.phases = new Phase(this, data, this.read('lessonLocation'));
    // Ggf. Hauptphase starten
    if (!this.phases.start())
        this.registerPhaseStarted(true);
    // Phasen anzeigen
    this.phases.render(nodeParent);
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
PhasesTask.prototype.connect = function() {
    if (typeof ScoTools.LMSLink == 'function') {
        var self = this;
        this.lmslink = new ScoTools.LMSLink();
        this.lmslink.onnew = (function() {
            self.lmsValuesInit();
        });
        this.lmslink.start();
    } else
        this.lmslink = { 'ready': false };
    this._lessonMode = this.lmslink.ready ? this.lmslink.lessonMode : 'normal';
    if (!this.lmslink.ready)
        this.showMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn');
    this.showFinalMessage();
}

/**
* Beim ersten Start des Lernpaketes Anfangswerte festlegen
* @return undefined
*/
PhasesTask.prototype.lmsValuesInit = function() {
    this.lmslink.save({
        "score":            0,
        "scoreMin":         0,
        "scoreMax":         100,
        "lessonLocation":   '',
        "lessonStatus":     'not atempted',
        "suspendData":      Math.floor((new Date()).getTime() / 1000)
    });
}

/**
* Anzeigen, falls Aufgabe endgültig erfolgreich abgeschlossen oder endgültig misslungen ist (sofern das nicht ohnehin schon angezeigt wird)
* @return undefined
*/
PhasesTask.prototype.showFinalMessage = function() {
    if (this._finalMessage)
        return;
    if (this.getDone()) {
        if (this.lmslink.lessonStatus == 'failed')
            this.showMsg('Aufgabe wurde engültig nicht geschafft', false);
        else
            this.showMsg('Aufgabe erfolgreich abgeschlossen', true);
        this._finalMessage = true;
    }
}

// Genutzte Instanz von LMSLink
PhasesTask.prototype.lmslink = { 'ready': false };

// Phase-Instanz mit Hauptphase
PhasesTask.prototype.phases = null;

// MessagePlace-Instanz für nicht-generelle Meldungen
PhasesTask.prototype.msgPlace = null;

// Zeit seit dem Start der letzten Phase
PhasesTask.prototype._time = (new Date()).getTime();

/**
* Eine Meldung anzeigen
* @return undefined
*/
PhasesTask.prototype.showMsg = function(
    text,           // Textinhalt der Meldung
    type            // Optional: Art der Meldung (false, true oder Wert aus ScoTools.MSGTYPES)
) {
    if (this.msgPlace)
        this.msgPlace.add(text, type);
}

/**
* Zuletzt angezeigte Meldung löschen
* @return undefined
*/
PhasesTask.prototype.clearMsg = function() {
    this.msgPlace.clear();
}

/**
* Prüft, ob eine Verbindung zur Lernplattform besteht
* @return Boolean   Besteht die Verbindung?
*/
PhasesTask.prototype.isConnected = function() {
    return this.lmslink.ready;
}

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
PhasesTask.prototype.getDone = function() {
    return (this.lmslink.ready && !this.lmslink.isIncomplete());
}

/**
* Bearbeitungszeit speichern
* @return undefined
*/
PhasesTask.prototype.reportTime = function() {
    this.lmslink.ready && this.lmslink.reportTime();
}

/**
* Einen Wert vom LMS lesen
* @return undefined
*/
PhasesTask.prototype.read = function(
    key     // Attributname von LMSLink-Instanz, die gelesen werden soll
) {
    return (this.lmslink.ready ? this.lmslink[key] : undefined);
}

/**
* Darauf reagieren, wenn eine neue Phase gestartet wurde
* @return undefined
*/
PhasesTask.prototype.registerPhaseStarted = function(
    noreset,        // Soll das Speichern der neuen Startzeit unterdrückt werden?
    timeoffset      // Um wie viele Sekunden soll die Startzeit korrigiert werden?
) {
    if (!noreset)
        this._time = (new Date()).getTime();
    if (Number.isFinite(timeoffset))
        this._time += timeoffset;
    if ((['browse', 'review'].indexOf(this._lessonMode) != -1) || this.getDone())
        return;
    var completed = this.phases.hasExited();
    if (this.lmslink.ready)
        this.lmslink.save({
            "score":            Math.min(100, Math.max(0, this.phases.countScore())),
            "scoreMin":         0,
            "scoreMax":         100,
            "lessonLocation":   this.phases.listDone(),
            "lessonStatus":     completed ? 'completed' : 'incomplete',
            "suspendData":      Math.floor(this._time / 1000)
        });
    if (completed) {
        this._lessonMode = 'review';
        this.showFinalMessage();
    }
}

/**
* Ermittelt, wie viele Sekunden der Start der letzten Phase zurückliegt
* @return Number    Anzahl an Sekunden (bzw. -1 im browse- und Infinity im review-Modus)
*/
PhasesTask.prototype.getTime = function() {
    switch (this._lessonMode) {
        case 'browse':
            return -1;
        case 'review':
            return Infinity;
        case 'normal':
        default:
            return ((new Date()).getTime() - this._time) / 1000;
    }
}
